unit lissajoux;

{----------------------------------------------------}
{ TLissajoux component for Delphi 3..7               }
{ Version: 1.2                                       }
{ Created: 2004.11.20                                }
{ Updated: 2012.09.15                                }
{ E-mail:  monster@Noniewicz.com                     }
{ WWW:     http://www.Noniewicz.com                  }
{ Legal:   (c)1992-2012 MoNsTeR/GDC, Jakub Noniewicz }
{ Licence: FREEWARE                                  }
{----------------------------------------------------}
{ History:                                           }
{ Version 1.0, last update: 2004.11.20               }
{ Version 1.0, last update: 2004.11.20               }
{ Version 1.1, last update: 2005.11.20               }
{ Version 1.2, last update: 2012.09.15               }
{----------------------------------------------------}
{                 LEGAL NOTICE:                      }
{ This software is provided 'as-is', without any     }
{ express or implied warranty. In no event will      }
{ the author be held liable for any damages arising  }
{ from the use of this software.                     }
{ Permission is granted to anyone to use this        }
{ software for any purpose, including commercial     }
{ applications, and to alter it and redistribute it  }
{ freely, subject to the following restrictions:     }
{ 1. The origin of this software must not            }
{    be misrepresented, you must not claim that you  }
{    wrote the original software.                    }
{    If you use this software in a product,          }
{    an acknowledgment in the product documentation  }
{    would be appreciated but is not required.       }
{ 2. Altered source versions must be plainly marked  }
{    as such, and must not be misrepresented as      }
{    being the original software.                    }
{ 3. This notice may not be removed or altered from  }
{    any source distribution.                        }
{----------------------------------------------------}


{todo:
- var fi+freq?
}

{History:
v1.2:
- minor changes
v1.1:
- paint to external canvas
- dotmode/linemode
- line width
}

interface

uses Windows, Classes, Graphics, Controls, Extctrls, Messages, SysUtils;

type
  TLissajoux = class(TCustomControl)
  private
    { Private declarations }
    FF1, FF2, FFi, FdT: double;
    FLineColor: TColor;
    FBGColor: TColor;
    FSteps: integer;
    FDotmode: boolean;
    FWidth: integer;
    procedure Paint; override;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure CWriteFLineColor(PLineColor: TColor);
    procedure CWriteFBGColor(PBGColor: TColor);
    procedure CWriteFF1(PF1: double);
    procedure CWriteFF2(PF2: double);
    procedure CWriteFFi(PFi: double);
    procedure CWriteFdT(PdT: double);
    procedure CWriteFSteps(PSteps: integer);
    procedure CWriteFDotmode(PDotmode: boolean);
    procedure CWriteFWidth(PWidth: integer);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure PaintExt(ExtCanvas: TCanvas; w, h: integer);
    procedure SaveToFile(FileName: string);
    procedure AssignToBitmap(var bmp: TBitmap);
  published
    { Published declarations }
    property LineColor: TColor read FLineColor write CWriteFLineColor;
    property BGColor: TColor read FBGColor write CWriteFBGColor;
    property Freq1: double read FF1 write CWriteFF1;
    property Freq2: double read FF2 write CWriteFF2;
    property Phase: double read FFi write CWriteFFi;
    property dT: double read FdT write CWriteFdT;
    property Steps: integer read FSteps write CWriteFSteps;
    property Dotmode: boolean read FDotmode write CWriteFDotmode;
    property LineWidth: integer read FWidth write CWriteFWidth;

    property ShowHint;
    property Hint;
    property Align;
    property Name;
    property Visible;
    property Tag;
    property Enabled;
    property Height;
    property Left;
    property Top;
    property Width;
    property Cursor;
    property PopupMenu;
  end;


procedure Register;


implementation


const rd = pi/180;


procedure Register;
begin
  RegisterComponents('Monster', [TLissajoux]);
end;

//---

constructor TLissajoux.Create(AOwner: TComponent);
begin
  inherited;

  FF1 := 19.0;
  FF2 := 31.0;
  FFi := 0;
  Fdt := 1;
  FSteps := 10000;
  FLineColor := RGB(0, 0, 0);
  FBGColor := clWhite;
  FDotmode := false;
  FWidth := 1; 

  self.Width := 320;
  self.Height := 256;
end; { Create }

destructor TLissajoux.Destroy;
begin
  inherited;
end; { Destroy }

//---

procedure TLissajoux.Paint;
begin
  inherited;
  PaintExt(self.Canvas, self.Width, self.Height);
end;

procedure TLissajoux.PaintExt(ExtCanvas: TCanvas; w, h: integer);
var i, x, y, ampli, sx, sy: integer;
    time, Freq, Fi: double;
begin
  Time := 0;
  if FF2 > 0 then
    Freq := FF1/FF2
  else
    Freq := 0; 
  Fi := FFi*pi;
  sx := w div 2;
  sy := h div 2;
  if sx > sy then ampli := sy-4 else ampli := sx-4;

  ExtCanvas.Brush.Color := FBGColor;
  ExtCanvas.Brush.Style := bsSolid;
  ExtCanvas.Pen.Color := FLineColor;
  ExtCanvas.Pen.Width := FWidth;
  ExtCanvas.FillRect(RECT(0, 0, w, h));

   for i := 1 to FSteps do
   begin
     x := round(sx+ampli*sin(rd*Time));
     y := round(sy+ampli*sin(rd*(Time*Freq+Fi)));
     if FDotmode then
       ExtCanvas.Pixels[x, y] := FLineColor
     else
     begin
       if i = 1 then
         ExtCanvas.MoveTo(x, y)
       else
         ExtCanvas.LineTo(x, y);
     end;
     Time := Time+dT;
   end;

end; { Paint }

procedure TLissajoux.WMSize(var Message: TWMSize);
begin
  self.Width := Message.Width;
  self.Height := Message.Height;
  Refresh;
end; { WMSize }

//---

procedure TLissajoux.CWriteFLineColor(PLineColor: TColor);
begin
  FLineColor := PLineColor;
  Refresh;
end;

procedure TLissajoux.CWriteFBGColor(PBGColor: TColor);
begin
  FBGColor := PBGColor;
  Refresh;
end; 

procedure TLissajoux.CWriteFF1(PF1: double);
begin
  FF1 := PF1;
  Refresh;
end;

procedure TLissajoux.CWriteFF2(PF2: double);
begin
  FF2 := PF2;
  Refresh;
end;

procedure TLissajoux.CWriteFFi(PFi: double);
begin
  FFi := PFi;
  Refresh;
end;

procedure TLissajoux.CWriteFdT(PdT: double);
begin
  FdT := PdT;
  Refresh;
end;

procedure TLissajoux.CWriteFSteps(PSteps: integer);
begin
  FSteps := PSteps;
  Refresh;
end;

procedure TLissajoux.CWriteFDotmode(PDotmode: boolean);
begin
  FDotmode := PDotmode;
  Refresh;
end;

procedure TLissajoux.CWriteFWidth(PWidth: integer);
begin
  FWidth := PWidth;
  Refresh;
end;

//---

procedure TLissajoux.SaveToFile(FileName: string);
var img: TBitmap;
    R: TRect;
begin
  if FileName <> '' then
  begin
    img := TBitmap.Create;
    img.Width := width;
    img.Height := height;
    R.Left := 0;
    R.Top := 0;
    R.Right := self.Width;
    R.Bottom := self.Height;
    img.Canvas.CopyRect(r, self.Canvas, r);
    try
      img.SaveToFile(FileName);
    except
    end;
    img.Free;
  end;
end;

procedure TLissajoux.AssignToBitmap(var bmp: TBitmap);
begin
  if assigned(bmp) then
  begin
    bmp.Width := width;
    bmp.Height := height;
    self.PaintTo(bmp.Canvas.Handle, 0, 0);
  end;
end; { AssignToBitmap }


end.
